﻿
Imports System.Collections
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms

Public Class Form1

    Private Matrix As Boolean(,)
    Private ListOfCurvesArray As ArrayList
    Private Bitmap As Bitmap
    Private factor As Single = 1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        Load_INI()
        StatusLabel1.Visible = False
        ProgressBar1.Visible = False
        lbl_ThresholdValue.Text = trackBar_Threshold.Value.ToString
        RasterToVectors.AreaMin = txt_AreaMin.NumericValueInteger
        RasterToVectors.CornerThreshold = txt_CornerThreshold.NumericValue
        RasterToVectors.ErrorTolerance = txt_ErrorTolerance.NumericValue
        btn_Vectorize.Enabled = False
        btn_Save.Enabled = False
        OpenImage(ImageFile)
        Me.Text = AppTitleAndVersion("Theremino - Img to Vectors")
        EventsAreEnabled = True
        Refresh()
        Opacity = 1
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Not EventsAreEnabled Then Exit Sub
        Save_INI()
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub

    Private Sub trackBar_Threshold_Scroll(ByVal sender As Object, ByVal e As EventArgs) Handles trackBar_Threshold.Scroll
        If Not EventsAreEnabled Then Return
        lbl_ThresholdValue.Text = trackBar_Threshold.Value.ToString
        RefreshMatrix()
        RefreshPicture()
    End Sub

    Private Sub trackBar_Zoom_Scroll(ByVal sender As Object, ByVal e As EventArgs) Handles trackBar_Size.Scroll
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub
    Private Sub Fillings_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles checkBox_Fillings.CheckedChanged
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub
    Private Sub Traces_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles checkBox_Traces.CheckedChanged
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub
    Private Sub Points_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles checkBox_Points.CheckedChanged
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub
    Private Sub ShowVector_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles radioButton_ShowVector.CheckedChanged
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub
    Private Sub ShowBitmap_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles radioButton_ShowBitmap.CheckedChanged
        If radioButton_ShowBitmap.Checked Then
            lbl_Threshold.Enabled = True
            lbl_ThresholdValue.Enabled = True
            trackBar_Threshold.Enabled = True
            lbl_size.Enabled = False
            lbl_SizeValue.Enabled = False
            trackBar_Size.Enabled = False
        Else
            lbl_Threshold.Enabled = False
            lbl_ThresholdValue.Enabled = False
            trackBar_Threshold.Enabled = False
            lbl_size.Enabled = True
            lbl_SizeValue.Enabled = True
            trackBar_Size.Enabled = True
        End If
        If Not EventsAreEnabled Then Return
        RefreshPicture()
    End Sub


    Private Sub RefreshPicture()
        If Matrix Is Nothing Then Return
        factor = (321.0F + 15.01F * trackBar_Size.Value) / Matrix.GetLength(0)

        Dim w As Int32 = CInt((Matrix.GetLength(0) - 2) * factor)
        Dim h As Int32 = CInt((Matrix.GetLength(1) - 2) * factor)
        lbl_SizeValue.Text = w.ToString & " x " & h.ToString

        If pictureBox1.Image IsNot Nothing Then pictureBox1.Image.Dispose()
        If radioButton_ShowBitmap.Checked Then
            factor = 1
            'pictureBox1.Width = CInt(B.Width * factor)
            'pictureBox1.Height = CInt(B.Height * factor)
            Dim B As Bitmap = RasterToVectors.BinaryBitmap_to_Bitmap(Matrix)
            pictureBox1.Image = B
        Else
            'pictureBox1.Width = B.Width
            'pictureBox1.Height = B.Height
            Dim B As New Bitmap(w, h, Imaging.PixelFormat.Format32bppArgb)
            pictureBox1.Image = B
            B.SetResolution(Bitmap.HorizontalResolution * factor, Bitmap.VerticalResolution * factor)
            Dim g As Graphics = Graphics.FromImage(pictureBox1.Image)
            g.Clear(Color.White)
            DrawListToGraphics(g, False)
        End If
    End Sub

    Private Sub RefreshMatrix()
        If Bitmap Is Nothing Then Return
        Matrix = RasterToVectors.Bitmap_to_BinaryBitmap(Bitmap, trackBar_Threshold.Value)
    End Sub

    Private Sub btn_Load_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Load.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.Filter = "Image files (*.jpeg;*.gif;*.png;*.bmp;*.tiff)|*.jpeg;*.jpg;*.gif;*.png;*.bmp;*.tiff;*.tif|All files (*.*)|*.*"
        If IO.File.Exists(ImageFile) Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(ImageFile)
            ofd.FileName = ImageFile
        Else
            ImageFile = Application.StartupPath & "\Vector Images\"
            ofd.InitialDirectory = IO.Path.GetDirectoryName(ImageFile)
            'ofd.FileName = ImageFile
        End If
        If ofd.ShowDialog() = DialogResult.OK Then
            If Bitmap IsNot Nothing Then Bitmap.Dispose()
            OpenImage(ofd.FileName)
        End If
    End Sub

    Private Sub OpenImage(ByVal newName As String)
        Try
            Bitmap = New Bitmap(newName)
            ImageFile = newName
            GroupBox1.Enabled = False
            GroupBox1.Refresh()
            ListOfCurvesArray = Nothing
            EventsAreEnabled = False
            radioButton_ShowBitmap.Checked = True
            RefreshMatrix()
            RefreshPicture()
            btn_Vectorize.Enabled = True
            btn_Save.Enabled = False
            GroupBox1.Enabled = True
            GroupBox1.Refresh()
            EventsAreEnabled = True
        Catch
        End Try
    End Sub

    Private Sub btn_Save_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Save.Click
        If ListOfCurvesArray Is Nothing Then Return
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Enhanced meta file (*.emf)|*.emf|" & _
                     "Jpeg image (*.jpg)|*.jpg|" & _
                     "Portable network graphics (*.png)|*.png"
        sfd.FileName = RemoveExtension(ImageFile)
        sfd.FilterIndex = SaveFormat
        sfd.AddExtension = True
        If sfd.ShowDialog() = DialogResult.OK Then
            SaveFormat = sfd.FilterIndex
            Select Case SaveFormat
                Case 1
                    Dim g As Graphics = Graphics.FromImage(pictureBox1.Image)
                    DrawGraphicsPathToMetafile(g, sfd.FileName)
                Case 2
                    SaveImage(pictureBox1.Image, sfd.FileName, "jpg", 100)
                Case 3
                    pictureBox1.Image.Save(sfd.FileName, System.Drawing.Imaging.ImageFormat.Png)
            End Select
        End If
    End Sub

    Friend Function RemoveExtension(ByVal str As String) As String
        On Error Resume Next
        Return IO.Path.GetDirectoryName(str) & "\" & IO.Path.GetFileNameWithoutExtension(str)
    End Function


    ' ========================================================================================================
    '  RASTER TO VECTORS
    ' ========================================================================================================
    Private Sub btn_Vectorize_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btn_Vectorize.Click
        If Bitmap Is Nothing Then Return
        EventsAreEnabled = False

        StatusLabel1.Visible = True
        ProgressBar1.Visible = True

        GroupBox1.Enabled = False
        GroupBox1.Refresh()
        Refresh()
        '
        RasterToVectors.AreaMin = txt_AreaMin.NumericValueInteger
        RasterToVectors.CornerThreshold = txt_CornerThreshold.NumericValue
        RasterToVectors.ErrorTolerance = txt_ErrorTolerance.NumericValue 'if <> 0 then replace Bezier-segments by single segment
        '
        ListOfCurvesArray = New ArrayList()
        Matrix = RasterToVectors.Bitmap_to_BinaryBitmap(Bitmap, trackBar_Threshold.Value)
        RasterToVectors.BynaryBitmap_ToListOfCurves(Matrix, ListOfCurvesArray)

        radioButton_ShowVector.Checked = True
        RefreshMatrix()
        RefreshPicture()
        '
        ProgressBar1.Value = 0
        StatusLabel1.Visible = False
        ProgressBar1.Visible = False

        btn_Save.Enabled = True
        GroupBox1.Enabled = True
        EventsAreEnabled = True
    End Sub

    Private Sub DrawListToGraphics(ByVal g As Graphics, ByVal Flatten As Boolean)
        If ListOfCurvesArray Is Nothing Then Return
        Dim gp As GraphicsPath = New GraphicsPath()
        For i As Integer = 0 To ListOfCurvesArray.Count - 1
            Dim CurveArray As ArrayList = DirectCast(ListOfCurvesArray(i), ArrayList)
            Dim Contour As GraphicsPath = Nothing
            Dim Hole As GraphicsPath = Nothing
            Dim Current As GraphicsPath = Nothing
            '
            For j As Integer = 0 To CurveArray.Count - 1
                If j = 0 Then
                    Contour = New GraphicsPath()
                    Current = Contour
                Else
                    Hole = New GraphicsPath()
                    Current = Hole
                End If
                Dim Curves As RasterToVectors.Curve() = DirectCast(CurveArray(j), RasterToVectors.Curve())

                For k As Integer = 0 To Curves.Length - 1
                    If Curves(k).Kind = RasterToVectors.CurveKind.Bezier Then
                        Current.AddBezier(CSng(Curves(k).A.x) * factor, _
                                          CSng(Curves(k).A.y) * factor, _
                                          CSng(Curves(k).ControlPointA.x) * factor, _
                                          CSng(Curves(k).ControlPointA.y) * factor, _
                                          CSng(Curves(k).ControlPointB.x) * factor, _
                                          CSng(Curves(k).ControlPointB.y) * factor, _
                                          CSng(Curves(k).B.x) * factor, _
                                          CSng(Curves(k).B.y) * factor)
                    Else
                        Current.AddLine(CSng(Curves(k).A.x) * factor, _
                                        CSng(Curves(k).A.y) * factor, _
                                        CSng(Curves(k).B.x) * factor, _
                                        CSng(Curves(k).B.y) * factor)
                    End If
                Next
                If j > 0 Then
                    Contour.AddPath(Hole, False)
                End If
            Next
            gp.AddPath(Contour, False)
        Next
        If Flatten Then
            gp.Flatten()
        End If
        If checkBox_Fillings.Checked Then
            g.FillPath(Brushes.Black, gp)
        End If
        If checkBox_Traces.Checked Then
            g.DrawPath(Pens.Red, gp)
        End If
        If checkBox_Points.Checked Then
            DrawPoints(g)
        End If
    End Sub

    Private Sub DrawPoints(ByVal g As Graphics)
        If ListOfCurvesArray Is Nothing Then Return
        For i As Integer = 0 To ListOfCurvesArray.Count - 1
            Dim CurveArray As ArrayList = DirectCast(ListOfCurvesArray(i), ArrayList)
            For j As Integer = 0 To CurveArray.Count - 1
                Dim Curves As RasterToVectors.Curve() = DirectCast(CurveArray(j), RasterToVectors.Curve())
                For k As Integer = 0 To Curves.Length - 1
                    g.FillRectangle(Brushes.Green, _
                                    CSng(((Curves(k).A.x) * factor - 1)), _
                                    CSng(((Curves(k).A.y) * factor - 1)), _
                                    2, 2)
                Next
            Next
        Next
    End Sub


    ' --------------------------------------------------------------------------------------
    '  metafile OK but not working on PSP7
    ' --------------------------------------------------------------------------------------
    Private Sub DrawGraphicsPathToMetafile(ByVal g As Graphics, ByVal MetaFile As String)
        Dim hdc As IntPtr = g.GetHdc()
        Dim myMetafile As Imaging.Metafile = New Imaging.Metafile(MetaFile, hdc)
        Dim myGraphics As Graphics = Graphics.FromImage(myMetafile)
        'myGraphics.Clear(pictureBox1.BackColor)
        ' ----------------------------------------------------------- draw
        DrawListToGraphics(myGraphics, False)
        'myGraphics.FillRectangle(Brushes.White, New RectangleF(0, 0, 5, 1))
        'myGraphics.FillRectangle(Brushes.Red, New RectangleF(0.25, 0.25, 4.5, 0.5))
        ' -----------------------------------------------------------
        myGraphics.Dispose()
        myMetafile.Dispose()
        g.ReleaseHdc(hdc)
    End Sub


    ' --------------------------------------------------------------------------------------
    '  trying to generate a good metafile (working on PSP7)
    ' --------------------------------------------------------------------------------------
    'Public Structure RECT
    '    Public Left As Integer
    '    Public Top As Integer
    '    Public Right As Integer
    '    Public Bottom As Integer
    'End Structure
    'Public Declare Auto Function CreateEnhMetaFile Lib "gdi32" (ByVal hdcRef As IntPtr, ByVal lpFilename As String, ByRef lpRect As RECT, ByVal lpDescription As String) As IntPtr
    'Public Declare Auto Function CloseEnhMetaFile Lib "gdi32" (ByVal hdc As IntPtr) As IntPtr
    'Public Declare Auto Function DeleteEnhMetaFile Lib "gdi32" (ByVal hemfbitHandle As IntPtr) As Boolean

    'Private Sub DrawGraphicsPathToMetafile(ByVal g As Graphics, ByVal MetaFile As String)
    '    Dim r As RECT
    '    r.Left = 0
    '    r.Top = 0
    '    r.Right = CInt(5.5 * 2540) ' 5 inches ( in .01 millimeters )
    '    r.Bottom = CInt(8.1 * 2540) ' 1 inch ( in .01 millimeters )

    '    Dim hdcMF As IntPtr = CreateEnhMetaFile(IntPtr.Zero, MetaFile, r, "")

    '    Dim gg As Graphics = Graphics.FromHdc(hdcMF)
    '    gg.PageUnit = GraphicsUnit.Inch
    '    gg.Clear(pictureBox1.BackColor)
    '    ' ----------------------------------------------------------- draw
    '    DrawListToGraphics(gg, True) ' <<< flattened for psp7

    '    gg.FillRectangle(Brushes.White, New RectangleF(0, 0, 5, 1))
    '    gg.FillRectangle(Brushes.Red, New RectangleF(0.25, 0.25, 4.5, 0.5))

    '    gg.Dispose()
    '    Dim hemf As IntPtr = CloseEnhMetaFile(hdcMF)
    '    DeleteEnhMetaFile(hemf)

    '    Diagnostics.Process.Start("""C:\Programmi\Jasc Software Inc\Paint Shop Pro 7\psp.exe""", """" & MetaFile & """")
    'End Sub


End Class